/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer*
* Copyright (C) 2013 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/**************************************************************************//**
* @file         ceu_sample.c
* @version      0.04
* $Rev: 11 $
* $Date:: 2013-09-05 15:58:35 +0900#$
* @brief        CEU Sample functions
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#ifdef _CEU_DEBUG_
#include    <stdio.h>
#endif

#include    <string.h>

#include    "ceu_sample.h"
#include    "r_ceu.h"
#include    "iodefine.h"
#include    "devdrv_intc.h"
#include    "dev_drv.h"


/******************************************************************************
Typedef definitions
******************************************************************************/
#define NULL 0
/******************************************************************************
Macro definitions
******************************************************************************/
#define BIT_00      (0x0001u)
#define BIT_01      (0x0002u)
#define BIT_02      (0x0004u)
#define BIT_03      (0x0008u)
#define BIT_04      (0x0010u)
#define BIT_05      (0x0020u)
#define BIT_06      (0x0040u)
#define BIT_07      (0x0080u)
#define BIT_08      (0x0100u)
#define BIT_09      (0x0200u)
#define BIT_10      (0x0400u)
#define BIT_11      (0x0800u)
#define BIT_12      (0x1000u)
#define BIT_13      (0x2000u)
#define BIT_14      (0x4000u)
#define BIT_15      (0x8000u)

#define BIT_ALLSET  (0xFFFFu)

#define DATA_SHIFT  (0x2u)

#ifdef _CEU_DEBUG_
#define R_INT_DRV_ERROR "ERROR!! R_INT_DRV\n\n"
#define R_CEU_DRV_ERROR "ERROR!! R_CEU_DRV\n\n"
#endif

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/
static void ceu_complete_callback( const uint32_t interrupt_flag );
static void ceu_init_func(void);

static volatile uint32_t    g_ceu_complete_flag;     /** operation complete flag */
static volatile uint32_t    g_ceu_error_flag;        /** error flag */


/**************************************************************************//**
 * Function Name: Graphics_CeuSingleCaptureInit
 * @brief       CEU module initialization.
 *
 *              Description:<br>
 *              In this function, the operations are performed as below.<br>
 *              - Initialization of a port.<br>
 *              - Initialization of module (module clock on).<br>
 *              - The configuration of a CEU module.<br>
 *              - Initialization of an interrupt controller.<br>
 *              - Registration of an interruption processing function.<br>
 *              - CEU interrupt enable.<br>
 *
 * @retval      none.
 *****************************************************************************/
void CEU_SAMPLE_CeuInit(int videoWidth, int videoHeight)
{
    int32_t ret_code;

    /* initialize port and module */
    R_CEU_ModuleInit( &ceu_init_func );

    /* CEU registers initialize for data enable capture mode. all interrupt is disabled. */

    CEU_ImageCaptureInit(videoWidth, videoHeight);

    /* ISR(Interrupt Service Rutine) registration */
    ret_code = R_INTC_RegistIntFunc(INTC_ID_CEUI, &CEU_Isr);
    if (DEVDRV_SUCCESS != ret_code)
    {
#ifdef _CEU_DEBUG_
    	(void)printf( R_INT_DRV_ERROR );
#endif
    }

    if( DEVDRV_SUCCESS == ret_code )
    {
		/* set priority */
		ret_code = R_INTC_SetPriority(INTC_ID_CEUI, (uint8_t)CEU_INTERRUPT_PRIORITY);
		if (DEVDRV_SUCCESS != ret_code)
		{
#ifdef CEU_DEBUG
			(void)printf( R_INT_DRV_ERROR );
#endif
		}
    }

    if( DEVDRV_SUCCESS == ret_code )
    {
		/* interrupt enable */
		ret_code = R_INTC_Enable(INTC_ID_CEUI);
		if (DEVDRV_SUCCESS != ret_code)
		{
#ifdef _CEU_DEBUG_
			(void)printf( R_INT_DRV_ERROR );
#endif
		}
    }

    if( DEVDRV_SUCCESS == ret_code )
    {
		/* CEU interrupt callback func set */
		R_CEU_CallbackISR( &ceu_complete_callback );

		/* interrupt enable for Data enable capture mode */
	//	R_CEU_InterruptEnable( CEU_INT_ONE_FRAME_CAPTUER_END );
    }

}   /* End of function Graphics_CeuSingleCaptureInit() */

/**************************************************************************//**
 * Function Name: CEU_SAMPLE_CeuSingleCaptureTerm
 * @brief       CEU module termination.
 *
 *              Description:<br>
 *              In this function, the operations are performed as below.<br>
 *              - CEU interrupt disable.<br>
 *
 * @retval      none.
 *****************************************************************************/
void CEU_SAMPLE_CeuSingleCaptureTerm(void)
{
	R_CEU_InterruptDisable();
}   /* End of function CEU_SAMPLE_CeuSingleCaptureTerm() */

/**************************************************************************//**
 * Function Name: Graphics_CeuSingleCapture
 * @brief       CEU Single Capture Start.
 *
 *              Description:<br>
 *              In this function, the operations are performed as below.<br>
 *              - Initialization of a variable.<br>
 *              - Start capture.<br>
 *              - Wait for capture end.<br>
 *
 * @param[out]  pCapBuff Pointer of capture buffer.
 * @retval      none.
 *****************************************************************************/
void CEU_SAMPLE_CeuSingleCapture( const uint16_t * const pCapBuff )
{
    ceu_error_t ceu_error;

    /* Initialization of a variable */
    g_ceu_complete_flag = 0;
    g_ceu_error_flag = 0;

    /* Destination memory address setting */
    ceu_error = R_CEU_SetMemoryAddress( (uint32_t)pCapBuff );
    if( ceu_error == CEU_ERROR_PARAM )
    {
#ifdef _CEU_DEBUG_
    	(void)printf( R_CEU_DRV_ERROR );
#endif
    }

    /* CEU Capture start */
    R_CEU_CaptureStart();

    /* wait capture operation complete */
    while ( CAPTURE_END != g_ceu_complete_flag )
    {
        /* wait */
    }

    if ( 0u != g_ceu_error_flag )
    {
#ifdef _CEU_DEBUG_
    	(void)printf( R_CEU_DRV_ERROR );
#endif
    }

    return;

}   /* End of function Graphics_CeuSingleCapture() */

/**************************************************************************//**
 * Function Name: Graphics_CeuConvert8bitRgbData
 * @brief       16bit --> 8bit data convert.
 *
 *              Description:<br>
 *              This function is converted to 8-bit data from 16-bit data.
 *
 *              CEU is 16bit mode. Camera (APTINA MT9V024) is 10bit bus width.
 *              capture data is as follows.<br>
 *              [16bit Data]<br>
 *              |----- 1 pixel ----| |----- 1 pixel ----| |----- 1 pixel ----| <br>
 *              0000 00XX XXXX XXxx, 0000 00XX XXXX XXxx, 0000 00XX XXXX XXxx,<br>
 *
 *              higher 8bit (bit[10:2]) is effective.<br>
 *
 *              [8bit Data]<br>
 *              | 1pixel | | 1pixel | | 1pixel | <br>
 *              XXXX XXXX, XXXX XXXX, XXXX XXXX,<br>
 *
 * @param[in]  pSrcBuff capture data (16bit bus width)
 * @param[out] pDstBuff VRAM buffer  (8bit data after conversion.)
 * @param[in]  stride   pDstBuff stride.
 * @return none.
 *****************************************************************************/
void CEU_SAMPLE_CeuConvertbitRgbData( const uint16_t * const pSrcBuff, uint8_t * const pDstBuff, const uint16_t stride )
{
	uint32_t    value;
    uint32_t    buff_height;
    uint32_t    buff_width;

    if( (pSrcBuff != NULL) && (pDstBuff != NULL) ){
		for ( buff_height = 0u; buff_height < CEU_CONFIG_CAPTURE_SCREEN_VSYNC_NUM_PXL; buff_height++ )
		{
			for ( buff_width = 0u ; buff_width < (uint32_t)stride ; buff_width++ )
			{
				if( buff_width < CEU_CONFIG_CAPTURE_SCREEN_HSYNC_NUM_PXL )
				{                                                                       /* DOUT[9:0] */
					value = (uint32_t)pSrcBuff[ ( CEU_CONFIG_CAPTURE_SCREEN_HSYNC_NUM_PXL * buff_height ) + buff_width ];
					value >>= DATA_SHIFT;                                               /* get DOUT[9:2] */
					pDstBuff[ ( (uint32_t)stride * buff_height ) + buff_width ] = (uint8_t)value; /* store DOUT[9:2] */
				}else{
					/* */
					pDstBuff[ ( (uint32_t)stride * buff_height ) + buff_width ] = (uint8_t)0u;   /* dummy */
				}
			}
		}
    }
    return;

}   /* End of function Graphics_CeuConvert8bitRgbData() */

/**************************************************************************//**
 * Function Name: ceu_complete_callback
 * @brief       CEU complete callback
 *
 *              Description:<br>
 *              The callback function for the completion of capture processing.
 * @param[in]   interrupt_flag CETCR register value.
 * @return none.
 *****************************************************************************/
static void ceu_complete_callback( const uint32_t interrupt_flag )
{
    g_ceu_complete_flag = CAPTURE_END;

    if ( (CEU_INT_ONE_FRAME_CAPTUER_END & interrupt_flag) != 0u )
    {
        g_ceu_error_flag = 0u;
    }
    else
    {
        /* sample program is not supporting this interrupt factor. */
        g_ceu_error_flag = interrupt_flag;
    }
    return;

}   /* End of function ceu_complete_callback() */

/**************************************************************************//**
 * Function Name: PekoeCeuPortInit
 * @brief       CEU port setting function.
 *
 *              Description:<br>
 *              Port initialize for CEU.<br>
 *              CMOS digital image sensor (OV7670) is 8bit buswidth.<br>
 * @return none.
 *****************************************************************************/

#ifdef PEKOE_BOARD
static void ceu_init_func( void )
{
	uint32_t reg_data;

	/* Port settings as follows.
	Pin	Port	Function	N function
	153	P1_4	VIO_D0		4
	154	P1_5	VIO_D1		4
	155	P1_6	VIO_D2		4
	156	P1_7	VIO_D3		4
	98	P1_12	VIO_D4		4
	99	P1_13	VIO_D5		4
	100	P1_14	VIO_D6		4
	101	P1_15	VIO_D7		4

	157	P3_15	VIO_FLD		2
	189	P4_1	VIO_HD		2
	188	P4_0	VIO_VD		2
	148	P3_14	VIO_CLK		2

	160 P2.8 	CAM_PWRDN	1	-> Input port
	161 P2.9 	CAM_RST		1	-> Input port   */

	GPIO.P2 &= (uint16_t)~( BIT_08 | BIT_09 );       /* P2.8=Low, P2.9=Low */

    // Port IP Control register (PIPC) PIPCnx : b'1 (SW input/output control)
	GPIO.PIPC1    |= (uint16_t)(BIT_15 | BIT_14 | BIT_13 | BIT_12 | BIT_07 | BIT_06 | BIT_05 | BIT_04 );
	GPIO.PIPC2    |= (uint16_t)( BIT_09 | BIT_08 );
	GPIO.PIPC3    |= (uint16_t)(BIT_14);
	GPIO.PIPC4    |= (uint16_t)(BIT_01 | BIT_00);

    // Por Mode register   PMnx : b=0 (output) PMnx=1 (input)
	GPIO.PM1    |= (uint16_t)(BIT_15 | BIT_14 | BIT_13 | BIT_12 | BIT_07 | BIT_06 | BIT_05 | BIT_04 );
	GPIO.PM2    |= (uint16_t)( BIT_09 | BIT_08 );
	GPIO.PM3    |= (uint16_t)(BIT_14);
	GPIO.PM4    |= (uint16_t)(BIT_01 | BIT_00);

    // Port Mode Control register PMCnx : b=0 (gpio mode) PMCnx : b=1 (function mode)
	GPIO.PMC1    |= (uint16_t)(BIT_15 | BIT_14 | BIT_13 | BIT_12 | BIT_07 | BIT_06 | BIT_05 | BIT_04 );
	GPIO.PMC2    &= (uint16_t)~( BIT_09 | BIT_08 ); //Force to port mode
	GPIO.PMC3    |= (uint16_t)(BIT_14);
	GPIO.PMC4    |= (uint16_t)(BIT_01 | BIT_00);



    /* set "Function6 (input)"
     * register setting is as follows.
     * PFCAE    PFCE   PFC    PM
     *  0        0      0      1  : Function1 (input)	<--- SET for P2
     *  0        0      1      1  : Function2 (input)   <--- SET for P3 & P4
     *  0        1      1      1  : Function4 (input)   <--- SET for P1     	*/

    // Port Function Control register (PFC) PFCn15 - PFCn0 : b=1 (function 4 input)
	GPIO.PFC1   |= (uint16_t)(BIT_15 | BIT_14 | BIT_13 | BIT_12 | BIT_07 | BIT_06 | BIT_05 | BIT_04 );
    // Port Function Control register (PFC) PFCn15 - PFCn0 : b=1 (function 2 input)
	GPIO.PFC3   |= (uint16_t)(BIT_14 );
    // Port Function Control register (PFC) PFCn15 - PFCn0 : b=1 (function 2 input)
	GPIO.PFC4    |= (uint16_t)( BIT_01 | BIT_00 );

    // Port Function Control Extend register (PFCE) PFCEn15 - PFCEn0 : b=1 (function 4 input)
	GPIO.PFCE1    |= (uint16_t)(BIT_15 | BIT_14 | BIT_13 | BIT_12 | BIT_07 | BIT_06 | BIT_05 | BIT_04 );
    // Port Function Control Extend register (PFCE) PFCEn15 - PFCEn0 : b=0 (function 2 input)
	GPIO.PFCE3    &=(uint16_t)~(BIT_14);
    // Port Function Control Extend register (PFCE) PFCEn15 - PFCEn0 : b=0 (function 2 input)
	GPIO.PFCE4   &=(uint16_t)~( BIT_01 | BIT_00 );

    // Port Function Control Additional Extend register (PFCAE) PFCAEn15 - PFCAEn0 : b=0 (function 2 input)
	GPIO.PFCAE1   &=(uint16_t) ~(BIT_15 | BIT_14 | BIT_13 | BIT_12 | BIT_07 | BIT_06 | BIT_05 | BIT_04 );
    // Port Function Control Additional Extend register (PFCAE) PFCAEn15 - PFCAEn0 : b=0 (function 2 input)
	GPIO.PFCAE3    &= (uint16_t)~(BIT_14);
    // Port Function Control Additional Extend register (PFCAE) PFCAEn15 - PFCAEn0 : b=0 (function 2 input)
	GPIO.PFCAE4    &=(uint16_t)~( BIT_01 | BIT_00 );

    // Port bi-directionally control register (PBDC) PBDCn15 : b=0 (disable)  PBDCn14 : b=1 (enable)
    GPIO.PBDC1 |= (uint16_t)(BIT_15 | BIT_14 | BIT_13 | BIT_12 | BIT_07 | BIT_06 | BIT_05 | BIT_04 );
	GPIO.PBDC2    |= (uint16_t)( BIT_09 | BIT_08 ); //Force to port mode
	GPIO.PBDC3    |= (uint16_t)(BIT_14);
	GPIO.PBDC4    |= (uint16_t)(BIT_01 | BIT_00);



    return;

}   /* End of function PekoeCeuPortInit() */
#else
static void ceu_init_func( void )
{
	uint32_t reg_data;

	/* P10 setting as follows.
     * P10_0 : VID_CLK(Function6)
     * P10_1 : VID_VD(Function6)
     * P10_2 : VID_HD(Function6)
     * P10_3 : GPIO(output Hi)   <-- RESET_BAR
     * P10_4 : VID_D0(Function6)
     * P10_5 : VID_D1(Function6)
     * P10_6 : VID_D2(Function6)
     * P10_7 : VID_D3(Function6)
     * P10_8 : VID_D4(Function6)
     * P10_9 : VID_D5(Function6)
     * P10_10 : VID_D6(Function6)
     * P10_11 : VID_D7(Function6)
     * P10_12 : VID_D8(Function6)
     * P10_13 : VID_D9(Function6)
     * P10_14 : GPIO(output Low) <-- VIO_D10(NC)
     * P10_15 : GPIO(output Low) <-- VIO_D11(NC)
     */
    /* Port Register
     * Pn3  : b'1 (hi level)
     * Pn15 : b'0 (low level)
     * Pn14 : b'0 (low level)
     * Pn13-0 : not change.
     */
	reg_data  = (uint32_t)GPIO.P10;
	reg_data &= ~( BIT_15 | BIT_14 );       /* P10.14=Low, P10.15=Low */
    GPIO.P10  = (uint16_t)reg_data;         /* P10.3=Hi */

	reg_data  = (uint32_t)GPIO.P10;
	reg_data |= ( BIT_03 );
    GPIO.P10 = (uint16_t)reg_data;          /* P10.3=Hi */

    /* Port IP Control register (PIPC)
     * PIPCn15 : b'0 (SW input/output control)
     * PIPCn14 : b'0 (SW input/output control)
     * PIPCn13 : b'0 (SW input/output control)
     * PIPCn12 : b'0 (SW input/output control)
     * PIPCn11 : b'0 (SW input/output control)
     * PIPCn10 : b'0 (SW input/output control)
     * PIPCn9  : b'0 (SW input/output control)
     * PIPCn8  : b'0 (SW input/output control)
     * PIPCn7  : b'0 (SW input/output control)
     * PIPCn6  : b'0 (SW input/output control)
     * PIPCn5  : b'0 (SW input/output control)
     * PIPCn4  : b'0 (SW input/output control)
     * PIPCn3  : b'0 (SW input/output control)
     * PIPCn2  : b'0 (SW input/output control)
     * PIPCn1  : b'0 (SW input/output control)
     * PIPCn0  : b'0 (SW input/output control)
     */
	reg_data    = (uint32_t)GPIO.PIPC10;
	reg_data    &= ~(BIT_ALLSET);
    GPIO.PIPC10 = (uint16_t)reg_data;

    /* Por Mode register
     * PMn15 : b'0 (output)
     * PMn14 : b'0 (output)
     * PMn13 : b'1 (input)
     * PMn12 : b'1 (input)
     * PMn11 : b'1 (input)
     * PMn10 : b'1 (input)
     * PMn9  : b'1 (input)
     * PMn8  : b'1 (input)
     * PMn7  : b'1 (input)
     * PMn6  : b'1 (input)
     * PMn5  : b'1 (input)
     * PMn4  : b'1 (input)
     * PMn3  : b'0 (output)
     * PMn2  : b'1 (input)
     * PMn1  : b'1 (input)
     * PMn0  : b'1 (input)
     */
	reg_data  = (BIT_13 | BIT_12 | BIT_11 | BIT_10 | BIT_09 |
                 BIT_08 | BIT_07 | BIT_06 | BIT_05 | BIT_04 |
                 BIT_02 | BIT_01 | BIT_00);
	GPIO.PM10 = (uint16_t)reg_data;

    /* Port Mode Control register
     * PMCn15 : b'0 (gpio mode)
     * PMCn14 : b'0 (gpio mode)
     * PMCn13 : b'1 (function mode)
     * PMCn12 : b'1 (function mode)
     * PMCn11 : b'1 (function mode)
     * PMCn10 : b'1 (function mode)
     * PMCn9  : b'1 (function mode)
     * PMCn8  : b'1 (function mode)
     * PMCn7  : b'1 (function mode)
     * PMCn6  : b'1 (function mode)
     * PMCn5  : b'1 (function mode)
     * PMCn4  : b'1 (function mode)
     * PMCn3  : b'0 (gpio mode)
     * PMCn2  : b'1 (function mode)
     * PMCn1  : b'1 (function mode)
     * PMCn0  : b'1 (function mode)
     */
	reg_data   = ( BIT_13 | BIT_12 | BIT_11 | BIT_10 | BIT_09 |
                   BIT_08 | BIT_07 | BIT_06 | BIT_05 | BIT_04 |
                   BIT_02 | BIT_01 | BIT_00);
    GPIO.PMC10 = (uint16_t)reg_data;

    /* set "Function6 (input)"
     * register setting is as follows.
     * PFCAE    PFCE   PFC    PM
     *  0        0      0      1  : Function1 (input)
     *  0        0      0      0  : Function1 (output)
     *  0        0      1      1  : Function2 (input)
     *  0        0      1      0  : Function2 (output)
     *  0        1      0      1  : Function3 (input)
     *  0        1      0      0  : Function3 (output)
     *  0        1      1      1  : Function4 (input)
     *  0        1      1      0  : Function4 (output)
     *  1        0      0      1  : Function5 (input)
     *  1        0      0      0  : Function5 (output)
     *  1        0      1      1  : Function6 (input)     <--- SET!!
     *  1        0      1      0  : Function6 (output)
     *  1        1      0      1  : Function7 (input)
     *  1        1      0      0  : Function7 (output)
     *  1        1      1      1  : Function8 (input)
     *  1        1      1      0  : Function8 (output)
     */

    /* Port Function Control register (PFC)
     * PFCn15 - PFCn0 : b'1 (function6 input)
     */
    GPIO.PFC10 = (uint16_t)BIT_ALLSET;

    /* Port Function Control Extend register (PFCE)
     * PFCEn15 - PFCEn0 : b'0 (function6 input)
     */
    GPIO.PFCE10 = (uint16_t)~(BIT_ALLSET);

    /* Port Function Control Additional Extend register (PFCAE)
     * PFCAEn15 - PFCAEn0 : b'1 (function6 input)
     */
    GPIO.PFCAE10 = (uint16_t)BIT_ALLSET;

    /* Port bi-directionally control register (PBDC)
     * PBDCn15 : b'0 (disable)
     * PBDCn14 : b'0 (disable)
     * PBDCn13 : b'0 (disable)
     * PBDCn12 : b'0 (disable)
     * PBDCn11 : b'0 (disable)
     * PBDCn10 : b'0 (disable)
     * PBDCn9  : b'0 (disable)
     * PBDCn8  : b'0 (disable)
     * PBDCn7  : b'0 (disable)
     * PBDCn6  : b'0 (disable)
     * PBDCn5  : b'0 (disable)
     * PBDCn4  : b'0 (disable)
     * PBDCn3  : b'0 (disable)
     * PBDCn2  : b'0 (disable)
     * PBDCn1  : b'0 (disable)
     * PBDCn0  : b'0 (disable)
     */
    GPIO.PBDC10 = (uint16_t)~(BIT_ALLSET);

    /* P11 setting as follows.
     * P11_0 : GPIO (low-level)  <-- VIO_D12(NC)
     * P11_1 : GPIO (low-level)  <-- VIO_D13(NC)
     * P11_2 : GPIO (low-level)  <-- VIO_D14(NC)
     * P11_3 : GPIO (low-level)  <-- VIO_D15(NC)
     */
    /* Port Register
     * Pn15-4 : not change.
     * Pn3  : b'0 (low level)
     * Pn2  : b'0 (low level)
     * Pn1  : b'0 (low level)
     * Pn0  : b'0 (low level)
     */
	reg_data = (uint32_t)GPIO.P11;
	reg_data &= ~( BIT_03 | BIT_02 | BIT_01 | BIT_00 );
    GPIO.P11 = (uint16_t)reg_data;

    /* Port IP Control register (PIPC)
     * PIPCn15-4 : not change
     * PIPCn3 : b'0 (SW input/output control)
     * PIPCn2 : b'0 (SW input/output control)
     * PIPCn1 : b'0 (SW input/output control)
     * PIPCn0 : b'0 (SW input/output control)
     */
	reg_data    = (uint32_t)GPIO.PIPC11;
	reg_data    &= ~( BIT_03 | BIT_02 | BIT_01 | BIT_00 );
    GPIO.PIPC11 = (uint16_t)reg_data;

    /* Por Mode register
     * PMn15-4 : not change
     * PMn3  : b'0 (output)
     * PMn2  : b'0 (output)
     * PMn1  : b'0 (output)
     * PMn0  : b'0 (output)
     */
	reg_data  = (uint32_t)GPIO.PM11;
	reg_data  &= ~( BIT_03 | BIT_02 | BIT_01 | BIT_00 );
    GPIO.PM11 = (uint16_t)reg_data;

    /* Port bi-directionally control register (PBDC)
     * PBDCn15-4 : not change
     * PBDCn3  : b'0 (disable)
     * PBDCn2  : b'0 (disable)
     * PBDCn1  : b'0 (disable)
     * PBDCn0  : b'0 (disable)
     */
	reg_data    = (uint32_t)GPIO.PBDC11;
	reg_data    &= ~( BIT_03 | BIT_02 | BIT_01 | BIT_00 );
    GPIO.PBDC11 = (uint16_t)reg_data;

    return;

}   /* End of function ceu_init_func() */
#endif
