/*
 * FreeRTOS+UDP V1.0.1 (C) 2013 Real Time Engineers ltd.
 * All rights reserved
 *
 * This file is part of the FreeRTOS+UDP distribution.  The FreeRTOS+UDP license
 * terms are different to the FreeRTOS license terms.
 *
 * FreeRTOS+UDP uses a dual license model that allows the software to be used
 * under a standard GPL open source license, or a commercial license.  The
 * standard GPL license (unlike the modified GPL license under which FreeRTOS
 * itself is distributed) requires that all software statically linked with
 * FreeRTOS+UDP is also distributed under the same GPL V2 license terms.
 * Details of both license options follow:
 *
 * - Open source licensing -
 * FreeRTOS+UDP is a free download and may be used, modified, evaluated and
 * distributed without charge provided the user adheres to version two of the
 * GNU General Public License (GPL) and does not remove the copyright notice or
 * this text.  The GPL V2 text is available on the gnu.org web site, and on the
 * following URL: http://www.FreeRTOS.org/gpl-2.0.txt.
 *
 * - Commercial licensing -
 * Businesses and individuals that for commercial or other reasons cannot comply
 * with the terms of the GPL V2 license must obtain a commercial license before
 * incorporating FreeRTOS+UDP into proprietary software for distribution in any
 * form.  Commercial licenses can be purchased from http://shop.freertos.org/udp
 * and do not require any source files to be changed.
 *
 * FreeRTOS+UDP is distributed in the hope that it will be useful.  You cannot
 * use FreeRTOS+UDP unless you agree that you use the software 'as is'.
 * FreeRTOS+UDP is provided WITHOUT ANY WARRANTY; without even the implied
 * warranties of NON-INFRINGEMENT, MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. Real Time Engineers Ltd. disclaims all conditions and terms, be they
 * implied, expressed, or statutory.
 *
 * 1 tab == 4 spaces!
 *
 * http://www.FreeRTOS.org
 * http://www.FreeRTOS.org/udp
 *
 */

/* Standard includes. */
#include <stdint.h>

/* FreeRTOS includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"
#include    "r_ether.h"

/* FreeRTOS+UDP includes. */
#include "FreeRTOS_UDP_IP.h"
#include "FreeRTOS_IP_Private.h"
#include "FreeRTOS_Sockets.h"
#include "NetworkBufferManagement.h"

/* Demo includes. */
#include "NetworkInterface.h"

/* Library includes. */
//#include "board.h"

/* The semaphore used to wake the deferred interrupt handler task when an Rx
interrupt is received. */
xSemaphoreHandle xEMACRxEventSemaphore = NULL;

/* The queue used to communicate Ethernet events to the IP task. */
extern xQueueHandle xNetworkEventQueue;
/*
 * A deferred interrupt handler task that processes received frames.
 */
static void prvEMACDeferredInterruptHandlerTask( void *pvParameters );

/*-----------------------------------------------------------*/

//static uint8_t MyMacAddr[] = { 0x00, 0x11, 0x22, 0x33, 0x44, 0x55 };
extern uint8_t ucMACAddress[ 6 ];
portBASE_TYPE xNetworkInterfaceInitialise( void )
{
	int errorCode;

    errorCode = R_Ether_Open( 0, &ucMACAddress[0] );
    if( errorCode != R_ETHER_OK )
    {
        R_Ether_Close( 0 );
        return pdFAIL;
    }

    /* Create the event semaphore if it has not already been created. */
	if( xEMACRxEventSemaphore == NULL )
	{
#if 0
		vSemaphoreCreateBinary( xEMACRxEventSemaphore );
#else
		xEMACRxEventSemaphore = xSemaphoreCreateCounting(10, 0);
#endif

		/* The Rx deferred interrupt handler task is created at the highest
		possible priority to ensure the interrupt handler can return directly to
		it no matter which task was running when the interrupt occurred. */

		xTaskCreate( 	prvEMACDeferredInterruptHandlerTask,/* The function that implements the task. */
						( const signed char * const ) "MACTsk",
						configMINIMAL_STACK_SIZE,	/* Stack allocated to the task (defined in words, not bytes). */
						NULL, 						/* The task parameter is not used. */
						configMAX_PRIORITIES - 1, 	/* The priority assigned to the task. */
						NULL );						/* The handle is not required, so NULL is passed. */
	}
	return pdPASS;
}


/*-----------------------------------------------------------*/

portBASE_TYPE xNetworkInterfaceOutput( xNetworkBufferDescriptor_t * const pxNetworkBuffer )
{
	int errorCode = R_ETHER_ERROR;

	while(1)
	{
		errorCode = R_Ether_Write( 0, (void *)pxNetworkBuffer->pucEthernetBuffer, (uint32_t)pxNetworkBuffer->xDataLength );
		if(errorCode == R_ETHER_OK) break;
	}

	/* Finished with the network buffer. */
	vNetworkBufferRelease( pxNetworkBuffer );

	return pdPASS;
}
/*-----------------------------------------------------------*/

static void prvEMACDeferredInterruptHandlerTask( void *pvParameters )
{
	xNetworkBufferDescriptor_t *pxNetworkBuffer;
	xIPStackEvent_t xRxEvent;
	int xBytesReceived;

	( void ) pvParameters;
	configASSERT( xEMACRxEventSemaphore );

	for( ;; )
	{
		/* Wait for the EMAC interrupt to indicate that another packet has been
		received.  The while() loop is only needed if INCLUDE_vTaskSuspend is
		set to 0 in FreeRTOSConfig.h.  If INCLUDE_vTaskSuspend is set to 1
		then portMAX_DELAY would be an indefinite block time and
		xSemaphoreTake() would only return when the semaphore was actually
		obtained. */
		xSemaphoreTake( xEMACRxEventSemaphore, portMAX_DELAY );

		pxNetworkBuffer = pxNetworkBufferGet( ipTOTAL_ETHERNET_FRAME_SIZE, ( portTickType ) 0 );

		if(pxNetworkBuffer != NULL)
		{
			xBytesReceived = R_Ether_Read( 0 , (void *) pxNetworkBuffer->pucEthernetBuffer);

			if( eConsiderFrameForProcessing( pxNetworkBuffer->pucEthernetBuffer ) == eProcessBuffer )
			{
				/* The event about to be sent to the IP stack is an Rx event. */
				xRxEvent.eEventType = eEthernetRxEvent;

				/* pvData is used to point to the network buffer descriptor that
				references the received data. */
				xRxEvent.pvData = ( void * ) pxNetworkBuffer;

				if( xQueueSendToBack( xNetworkEventQueue, &xRxEvent, ( portTickType ) 0 ) == pdFALSE )
				{
					/* The buffer could not be sent to the IP task so the
					buffer must be released. */
					vNetworkBufferRelease( pxNetworkBuffer );
					iptraceETHERNET_RX_EVENT_LOST();
				}
				else
				{
					iptraceNETWORK_INTERFACE_RECEIVE();
				}
			}
            else
            {
                /* The Ethernet frame can be dropped, but the Ethernet buffer
                must be released. */
                vNetworkBufferRelease( pxNetworkBuffer );
            }
		}
		else
		{
			iptraceETHERNET_RX_EVENT_LOST();
		}

	}
}

/*-----------------------------------------------------------*/
